require("AllSigns.nut");

class SignProtector
{
	signs = {};
	
	constructor()
    {
        this.signs = this.GetAllSigns();
    }
	
	function GetAllSigns();
	function ProtectAllSigns();
	function AddAllNewSigns();
	function RemoveAllRemovedSigns();
	function UpdateExistingSigns();
}

function SignProtector::GetAllSigns()
{
	local signs = {};
	
	for(local companyID = GSCompany.COMPANY_FIRST; companyID <= GSCompany.COMPANY_LAST; companyID++) 
	{
		local resolvedCompanyID = GSCompany.ResolveCompanyID(companyID);
		if(resolvedCompanyID != GSCompany.COMPANY_INVALID)
		{
			local mode = GSCompanyMode(resolvedCompanyID);
			local signList = AllSigns();
			
			if (!signList.IsEmpty()) 
			{
				local sign = signList.GetFirstSign();
				do 
				{
					signs[sign.signID] <- sign;
				
					sign = signList.GetNextSign();
				} while(!signList.IsEnd());
			}
		}
	}
	
	return signs;
}

function SignProtector::ProtectAllSigns()
{
	local allCurrentSigns = this.GetAllSigns();
	
	this.AddAllNewSigns(allCurrentSigns);
	this.RemoveAllRemovedSigns(allCurrentSigns);
	this.UpdateExistingSigns(allCurrentSigns);
}

function SignProtector::AddAllNewSigns(allCurrentSigns)
{
	foreach (currentSign in allCurrentSigns)
	{
		if (!this.signs.rawin(currentSign.signID))
			signs[currentSign.signID] <- currentSign;
	}
}

function SignProtector::RemoveAllRemovedSigns(allCurrentSigns)
{
	foreach (savedSign in this.signs)
	{
		if (!allCurrentSigns.rawin(savedSign.signID))
			this.signs.rawdelete(savedSign.signID);
	}
}

function SignProtector::UpdateExistingSigns(allCurrentSigns)
{	
	foreach (currentSign in allCurrentSigns)
	{
		local savedSign = this.signs[currentSign.signID];
		
		if (savedSign.IsTextChanged(currentSign))
		{
			if (savedSign.IsSameOwner(currentSign))
				savedSign.SetChangesSaved(currentSign);
			else
			{
				local warningGoalID = GSGoal.Question(0, currentSign.ownerCompanyID, GSText(GSText.STR_SignProtector_UpdateExistingSigns_CannotChangeForeignSign), GSGoal.QT_WARNING, GSGoal.BUTTON_CLOSE);
				savedSign.RevertCurrentChanges(currentSign);
			}
		}
	}
}