require("AllVehicles.nut");
require("HighwaySigns.nut");
require("Payment.nut");

class HighwayTollClass extends GSController
{
	highwaySigns = null;
	mode = 1;
	
    function Start();
	function CheckAllVehicles();
	function PayOnAllForeignRoads();
	function PayFromSignToSignOnForeignRoads();
	function PayFromSignToSignOnAllRoads();
}

function HighwayTollClass::Start()
{
    this.Sleep(1); // Wait for the game to start

	this.mode = GSController.GetSetting("mode");
	this.highwaySigns = HighwaySigns();
	
    while (true)
	{
		if (this.mode != 0)
			this.highwaySigns.ProtectAllSigns();
		
		this.CheckAllVehicles();
		
		// Run every 0.5 day
        local delay_time = 32;
        this.Sleep(delay_time);
    }
}

function HighwayTollClass::CheckAllVehicles()
{
	local vehicleList = AllVehicles();
	if (!vehicleList.IsEmpty()) 
	{
		local vehicle = vehicleList.GetFirstVehicle();
		do 
		{
			switch(this.mode)
			{
				case 0:
					this.PayOnAllForeignRoads(vehicle);
					break;
				case 1:
					this.PayFromSignToSignOnForeignRoads(vehicle);
					break;
				case 2:
					this.PayFromSignToSignOnAllRoads(vehicle);
					break;
			}
		
			vehicle = vehicleList.GetNextVehicle();
		} while(!vehicleList.IsEnd());
	}
}

function HighwayTollClass::PayOnAllForeignRoads(vehicle)
{
	if (vehicle.IsRoadVehicle() && vehicle.IsRunningVehicle())
	{
		local payment = vehicle.OnForeignRoad();
		if (payment != null)
		{
			local paymentAmount = GSController.GetSetting("fee_per_day");
			payment.Pay(paymentAmount);
		}
	}
}

function HighwayTollClass::PayFromSignToSignOnForeignRoads(vehicle)
{
	if (this.highwaySigns.IsOnHighway(vehicle))
	{
		local vehicleTileID = vehicle.GetLocationTileID();
		local tileOwnerCompanyID = GSTile.GetOwner(vehicleTileID);

		if (tileOwnerCompanyID != GSCompany.COMPANY_INVALID)
		{
			local vehicleOwnerCompanyID = GSVehicle.GetOwner(this.vehicleID);
			
			if (tileOwnerCompanyID != vehicleOwnerCompanyID)
			{
				local payment = Payment(vehicleOwnerCompanyID, tileOwnerCompanyID);
				local paymentAmount = GSController.GetSetting("fee_per_day");
				payment.Pay(paymentAmount);
			}
		}
	}
}

function HighwayTollClass::PayFromSignToSignOnAllRoads(vehicle)
{
	if (this.highwaySigns.IsOnHighway(vehicle))
	{
		local vehicleTileID = vehicle.GetLocationTileID();
		local tileOwnerCompanyID = GSTile.GetOwner(vehicleTileID);
		local vehicleOwnerCompanyID = GSVehicle.GetOwner(vehicle.vehicleID);
		
		local payment = null;
		
		if (tileOwnerCompanyID == GSCompany.COMPANY_INVALID || tileOwnerCompanyID == vehicleOwnerCompanyID)
			payment = Payment(vehicleOwnerCompanyID, GSCompany.COMPANY_INVALID);
		else
			payment = Payment(vehicleOwnerCompanyID, tileOwnerCompanyID);
		
		local paymentAmount = GSController.GetSetting("fee_per_day");
		payment.Pay(paymentAmount);
	}
}